<?php

/**
 * @file
 * menu callback for upload form generation and overview page
 */

/**
 * Template upload form definition
 */
function odt_filler_upload_form($form, &$form_state) {
  if (isset($form_state['storage']['odt_tpl'])) {
    // If the template upload form has been successfully submitted
    // return the association form.
    return odt_filler_association_form($form, $form_state);
  }
  $types = node_type_get_types();
  // Define the options array containing the available content types.
  foreach ($types as $type) {
    $options[$type->type] = $type->name;
  }
  $form['tpl_name'] = array(
    '#title' => t('Template Name'),
    '#type' => 'textfield',
    '#size' => 25,
    '#required' => TRUE,
  );
  $form['tpl_description'] = array(
    '#title' => t('Template Description'),
    '#type' => 'textarea',
    '#rows' => 3,
    '#description' => t('Insert a description for this template'),
  );
  $form['tpl_button_name'] = array(
    '#title' => t('Button name'),
    '#type' => 'textfield',
    '#description' => t('Insert a label for the ODT creation button'),
    '#size' => 15,
    '#required' => TRUE,
  );
  $form['upload_tpl'] = array(
    '#title' => t('Upload ODT template'),
    '#type' => 'file',
    '#description' => t('Only odt file allowed'),
  );
  $form['tpl_content_type'] = array(
    '#type' => 'select',
    '#title' => t('Content Type to associate'),
    '#description' => t('Select the content type to associate with this ODT template'),
    '#options' => $options,
    '#required' => TRUE,
  );
  $form['submit'] = array(
    '#value' => t('Continue'),
    '#type' => 'submit',
    '#validate' => array('odt_filler_upload_form_validate'),
    '#submit' => array('odt_filler_upload_form_submit'),
  );
  return $form;
}

/**
 * Template variables => content type field association form definition
 */
function odt_filler_association_form($form, &$form_state, $edit = NULL) {

  // Check if it is an edit form.
  $odt_tpl_info = isset($edit) ? $edit : $form_state['storage']['odt_tpl'];

  // Recovery all field instances for the selected content type
  // and define the options array for the select
  // to show beside the template variable.
  $fields = field_info_instances('node', $odt_tpl_info['tpl_ct']);
  $options['title'] = t('Title');
  foreach ($fields as $key => $field) {
    // Show only the fields having text or number widget
    // todo: implement other field types.
    if ($field['widget']['module'] == 'text' || $field['widget']['module'] == 'number') {
      $options[$key] = t(check_plain($field['label']));
    }
  }
  foreach ($odt_tpl_info['tpl_vars'] as $key => $tpl_var) {
    // Create an array to track how many variables exist.
    $keys[] = $key;

    $form['tpl_var'][$key] = array(
      '#markup' => isset($edit) ? $key : $tpl_var,
    );
    $form['tpl_ct_field'][$key] = array(
      '#type' => 'select',
      '#options' => $options,
      '#required' => TRUE,
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#validate' => array('odt_filler_association_form_validate'),
    '#submit' => array('odt_filler_association_form_submit'),
  );
  $form['#theme'] = 'odt_filler_association_form';
  $form_state['storage']['odt_tpl']['keys'] = $keys;
  return $form;
}

/**
 * Upload form validate function
 */
function odt_filler_upload_form_validate($form, &$form_state) {
  // Check the library existence and include it.
  if (!odt_filler_check_library()) {
    form_set_error(t('Library missed'));
  }
  $path_tpl = variable_get('odt_filler_tpl_path', 'sites/default/files/odt_filler_tpl');

  // Validate only odt files.
  $file = file_save_upload('upload_tpl', array('file_validate_extensions' => array('odt')));
  if ($file) {
    // Check template folder existence and create it if it not exists.
    if (file_prepare_directory($path_tpl, FILE_MODIFY_PERMISSIONS | FILE_CREATE_DIRECTORY)) {
      $path_tpl_uri = 'odtfiller://';

      // Move the uploaded file to the right folder
      // don't move it if it already exists.
      if (!is_file($path_tpl_uri . $file->filename)) {
        $file = file_move($file, $path_tpl_uri);

        // Save in storage the moved file object and the path.
        $form_state['storage']['file'] = $file;
        $form_state['storage']['odt_tpl']['tpl_path'] = $path_tpl;
      }
      else {
        $fid = db_select('file_managed', 'fm')
          ->fields('fm', array('fid'))
          ->condition('filename', $file->filename)
          ->execute()
          ->fetchField();
        file_delete($file);
        $file = file_load($fid);
        $form_state['storage']['file'] = $file;
        $form_state['storage']['odt_tpl']['tpl_path'] = $path_tpl;
        drupal_set_message(t('The selected file already exists. The existing file will be used.'), 'warning');
      }
    }
    else {
      form_set_error('upload_tpl', t('The folder for template files has not been created'));
    }
  }
  else {
    form_set_error('upload_tpl', t('No file was uploaded'));
  }
  // Check if the selected content type has field instances.
  if (!field_info_instances('node', $form_state['values']['tpl_content_type'])) {
    form_set_error('content_type', t("The selected content type doesn't contain fields, please select another one"));
  }
}

/**
 * Association form validate function
 */
function odt_filler_association_form_validate($form, &$form_state) {
  $i = 0;
  foreach (element_children($form['tpl_ct_field']) as $key => $field) {
    isset($form_state['values'][$key]) ? '' : $i++;
  }
  ($i > 0) ? form_set_error('tpl_ct_field', t('You need to associate every template variable!')) : '';
}

/**
 * Upload form submit function
 */
function odt_filler_upload_form_submit($form, &$form_state) {
  // Generate the odt template path.
  $odt_path = $form_state['storage']['odt_tpl']['tpl_path'] . '/' . $form_state['storage']['file']->filename;

  // Define configuration array for the odf object instance.
  $config = array(
    'ZIP_PROXY' => variable_get('odt_filler_zip_proxy', 'PhpZipProxy'),
    'PATH_TO_TMP' => variable_get('file_temporary_path'),
  );

  // Create a new odf object passing the early uploaded template.
  $odf = new odf($odt_path, $config);

  // Recovery the odt content.
  $contentxml = $odf->__toString();

  // Match all the template variables (having the following form: {var}).
  preg_match_all('/{([^{]+?)}/', $contentxml, $matches);

  // Working on the regexp to match the table rows starting and ending chars
  // in odt template.
  // '/\[!-- BEGIN.*\.(.*) --\](.*)\[!-- END.*--\]/'.
  // Save all info into the storage bin.
  $form_state['storage']['odt_tpl'] = array(
    'tpl_vars' => $matches[1],
    'tpl_name' => $form_state['values']['tpl_name'],
    'tpl_description' => $form_state['values']['tpl_description'],
    'tpl_ct' => $form_state['values']['tpl_content_type'],
    'tpl_button_name' => $form_state['values']['tpl_button_name'],
  );
  $form_state['rebuild'] = TRUE;
}

/**
 * Association form submit function
 */
function odt_filler_association_form_submit($form, &$form_state) {
  // Generate an array into the storage bin in the form
  // 'tpl_variable => content_type_field'.
  foreach ($form_state['storage']['odt_tpl']['keys'] as $key) {
    $form_state['storage']['odt_tpl']['tpl_vars_fields'][$form_state['storage']['odt_tpl']['tpl_vars'][$key]] = $form_state['values'][$key];
  }
  $file = $form_state['storage']['file'];

  // Set file as permanent and save it.
  $file->status = FILE_STATUS_PERMANENT;
  file_save($file);

  // Serialize the variables/ct_fields array to store it into the db.
  $tpl_vars_serial = serialize($form_state['storage']['odt_tpl']['tpl_vars_fields']);

  // Prepare fields for the db_insert query.
  $fields = array(
    'fid' => $form_state['storage']['file']->fid,
    'ct' => $form_state['storage']['odt_tpl']['tpl_ct'],
    'tpl_vars' => $tpl_vars_serial,
    'uid' => $form_state['storage']['file']->uid,
    'tpl_name' => $form_state['storage']['odt_tpl']['tpl_name'],
    'description' => $form_state['storage']['odt_tpl']['tpl_description'],
    'button_name' => $form_state['storage']['odt_tpl']['tpl_button_name'],
  );
  $query = db_insert('odt_filler_tpl')
    ->fields($fields);
  if ($query->execute()) {
    drupal_set_message(t('The ODT template has been saved succesfully'));
  }
  else {
    drupal_set_message(t('Something went wrong, please retry'), 'error');
  }

  // Unset storage bin to prevent the form to be rebuilt.
  unset($form_state['storage']);

  // Redirect the user to the overview page.
  $form_state['redirect'] = 'admin/structure/odt_filler/overview';
}

/**
 * Association form theming function
 */
function theme_odt_filler_association_form($variables) {
  $form = $variables['form'];
  $header = array(t('Template Variable'), t('Content Type Field'));
  $rows = array();
  $output = '';

  // Check if the form is an edit one.
  $is_edit = (isset($form['tpl_main_info'])) ? TRUE : FALSE;
  if ($is_edit) {

    // If it's an edit form render first the main part of it.
    $output .= drupal_render($form['tpl_main_info']);
  }

  // Cicle over the non-property elements of the form element array.
  foreach (element_children($form['tpl_var']) as $key) {
    $row = array();
    $row[] = drupal_render($form['tpl_var'][$key]);
    $row[] = drupal_render($form['tpl_ct_field'][$key]);
    $rows[] = $row;
  }

  // Produce different output if the form is an edit one or not.
  $output .= ($is_edit) ? '' : '<h2>' . t('Template variable association') . '</h2>';
  $output .= ($is_edit) ? '<label for="tpl_vars_fields">' . t('Variables/Fields association') . '</label>' : '<p>' . t('Associate the ODT template variable below to the previously selected content type fields') . '</p>';
  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'tpl_vars_fields')));
  $output .= drupal_render_children($form);
  return $output;
}

/**
 * Overview table form function
 */
function odt_filler_overview($form, &$form_state) {
  // Check if is needed to return a confirm form to delete templates.
  if (isset($form_state['storage']['confirm_del'])) {
    $form['overview'] = array(
      '#type' => 'value',
      '#value' => $form_state['storage']['tpls'],
    );
    $form = confirm_form($form, t('Are you sure you want remove the selected templates?'), $_GET['q']);
    return $form;
  }

  // Prepare the header for the tableselect overview form.
  $header = array(
    'tpl_name' => array('data' => 'Name', 'field' => 'tpl_name'),
    'tpl_description' => array('data' => t('Description'), 'field' => 'description'),
    'tpl_ct' => array('data' => t('Content Type'), 'field' => 'ct'),
    'tpl_file' => array('data' => t('Filename'), 'field' => 'fid'),
    'tpl_user' => array('data' => t('User'), 'field' => 'uid'),
    'tpl_ops' => array('data' => t('Operations'),),
  );

  // Get all info from the odt_filler_tpl table.
  $results = db_select('odt_filler_tpl', 'oft')
    ->fields('oft', array('id', 'fid', 'ct', 'uid', 'tpl_name', 'description'))
    ->extend('PagerDefault')
      ->limit(10)
    ->extend('TableSort')
      ->orderByHeader($header)
    ->execute()
    ->fetchAllAssoc('id');
  $options = array();

  // Cicle over the query results to produce
  // the rows of the tableselect overview form.
  foreach ($results as $tpl_id => $result) {
    $operations['edit'] = array(
      'title' => t('Edit'),
      'href' => 'admin/structure/odt_filler/edit/' . $tpl_id,
      'query' => drupal_get_destination(),
    );
    $file = file_load($result->fid);
    $user = user_load($result->uid);

    $options[$tpl_id] = array(
      'tpl_name' => check_plain($result->tpl_name),
      'tpl_description' => check_plain($result->description),
      'tpl_user' => check_plain($user->name),
      'tpl_ct' => check_plain(drupal_ucfirst($result->ct)),
      'tpl_file' => check_plain($file->filename),
      'tpl_ops' => array(
        'data' => array(
          '#theme' => 'links__node_operations',
          '#links' => $operations,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      ),
    );
  }
  $form['overview'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No ODT template available'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  return $form;
}

/**
 * Overview form validate function
 */
function odt_filler_overview_validate($form, &$form_state) {
  if (!isset($form_state['storage']['confirm_del'])) {
    if (!is_array($form_state['values']['overview']) || !count(array_filter($form_state['values']['overview']))) {
      form_set_error('', t('No items selected.'));
    }
  }
}

/**
 * Overview form submit function
 */
function odt_filler_overview_submit($form, &$form_state) {
  // Reject the non selected templates.
  $tpls = array_filter($form_state['values']['overview']);

  // Mark the form to present the delete operation confirm form.
  if (!isset($form_state['storage']['confirm_del'])) {
    $form_state['storage']['confirm_del'] = TRUE;
    $form_state['storage']['tpls'] = $tpls;
    $form_state['rebuild'] = TRUE;
    return;
  }
  foreach ($tpls as $tpl) {
    // Check if the same file is used by other templates
    // otherwise delete it.
    $query_fid = db_select('odt_filler_tpl', 'oft')
      ->fields('oft', array('fid'))
      ->condition('id', $tpl)
      ->execute()
      ->fetch();
    $count_fid = db_select('odt_filler_tpl', 'oft')
      ->fields('oft')
      ->condition('fid', $query_fid->fid)
      ->countQuery()
      ->execute()
      ->fetch();
    if ($count_fid->expression == 1) {
      $file = file_load($query_fid->fid);
      if (file_delete($file)) {
        drupal_set_message(t('ODT template %file deleted', array('%file' => $file->filename)));
      }
    }

    // Delete the row regarding the selected template in od_filler_tpl table.
    $query_del = db_delete('odt_filler_tpl')
      ->condition('id', $tpl)
      ->execute();
    if ($query_del) {
      drupal_set_message(t('ODT template successfully deleted'));
    }

    // Unset storage bin to prevent form rebuilding.
    unset($form_state['storage']);
  }
}

/**
 * Template edit form definition
 */
function odt_filler_edit_tpl($form, &$form_state, $tpl_id) {
  // Collect all info regarding the template id passed by the $tpl_id variable.
  $results = db_select('odt_filler_tpl', 'oft')
    ->fields('oft')
    ->condition('id', $tpl_id)
    ->execute()
    ->fetch();

  // Call upload form and association form functions to get the needed elements
  // and set their default values.
  $form = odt_filler_upload_form($form, $form_state);
  $form['tpl_name']['#default_value'] = check_plain($results->tpl_name);
  $form['tpl_description']['#default_value'] = check_plain($results->description);
  $form['tpl_button_name']['#default_value'] = check_plain($results->button_name);
  unset($form['upload_tpl'], $form['tpl_content_type'], $form['submit']);
  $keys = array('tpl_main_info');
  $form = array_combine($keys, array($form));
  $edit = array(
    'tpl_ct' => $results->ct,
    'tpl_vars' => unserialize($results->tpl_vars),
  );
  $form = odt_filler_association_form($form, $form_state, $edit);
  foreach ($edit['tpl_vars'] as $tpl_var => $field) {
    $form['tpl_ct_field'][$tpl_var]['#default_value'] = $field;
  }
  unset($form['submit']['#validate'], $form['submit']['#submit']);

  // Store result set into the storage bin for using it in the submit function.
  $results->tpl_vars = $edit['tpl_vars'];
  $form_state['storage']['tpl_row'] = $results;
  return $form;
}

/**
 * Edit template form submit function
 */
function odt_filler_edit_tpl_submit($form, &$form_state) {
  // Set an index to track if any template_variable/content_type_field
  // association changed.
  $i = 0;
  foreach ($form_state['storage']['tpl_row']->tpl_vars as $tpl_var => $field) {
    if ($field != $form_state['values'][$tpl_var]) {

      // If changes occured replace the $field in the result set.
      $form_state['storage']['tpl_row']->tpl_vars[$tpl_var] = $form_state['values'][$tpl_var];
      ++$i;
    }
  }

  // Check if changes occured in the other fields
  // and save them in the $fields array.
  foreach (element_children($form['tpl_main_info']) as $field) {
    if ($form_state['storage']['tpl_row']->$field != $form_state['values'][$field]) {
      $fields[$field] = $form_state['values'][$field];
    }
  }
  ($i > 0) ? $fields['tpl_vars'] = serialize($form_state['storage']['tpl_row']->tpl_vars) : '';
  if (isset($fields)) {
    // If the $fields array has been set execute the database update.
    $query_upd = db_update('odt_filler_tpl')
      ->fields($fields)
      ->condition('id', $form_state['storage']['tpl_row']->id)
      ->execute();
    if ($query_upd) {
      drupal_set_message(t('Template successfully updated'));
    }
  }
}
